/*	injection_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "injection_isrs.h"

/* Summary of intended injection timing scheme
 * 
 * Set ECT timer based on a reference point (probably single cam trigger or optionally toggle bit based on missing tooth on crank)
 * Set time to open for each channel based on the nearest engine position information and the current value of the timer at that signal (a particular edge of a particular crank tooth)
 * Injector turns on and generates interrupt, period of injection in timer units calculated from period of injection and injection start time.
 * Injector turns off and generates an interrupt, ignore that interrupt
 * Repeat this for all injectors each engine cycle
 */

/* Each interrupt handler should :
 * 
 * 1,2	Clear its interrupt flag, Disable all interrupts (or vice versa?)
 * 3	Check to see if its just opened or just closed
 * 4	If just opened, set the period of the cycle and the action to switch off
 * 5	If just closed set the action to switch on (and maybe clean up?)
 * 6	Return
 */

// TODO find a better way to reduce code duplication here!!

void Injector1ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN1;

	/* If rising edge triggered this */
	if(PTIT & MAIN1){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN1REG &= MAIN1GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED1ON;
			stagedOn |= STAGED1ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN1REG |= MAIN1GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED1ON){
			STAGEDPORT &= STAGED1OFF;
			stagedOn &= STAGED1OFF;
		}
	}
}

void Injector2ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN2;

	/* If rising edge triggered this */
	if(PTIT & MAIN2){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN2REG &= MAIN2GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED2ON;
			stagedOn |= STAGED2ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN2REG |= MAIN2GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED2ON){
			STAGEDPORT &= STAGED2OFF;
			stagedOn &= STAGED2OFF;
		}
	}
}

void Injector3ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN3;

	/* If rising edge triggered this */
	if(PTIT & MAIN3){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN3REG &= MAIN3GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED3ON;
			stagedOn |= STAGED3ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN3REG |= MAIN3GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED3ON){
			STAGEDPORT &= STAGED3OFF;
			stagedOn &= STAGED3OFF;
		}
	}
}

void Injector4ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN4;

	/* If rising edge triggered this */
	if(PTIT & MAIN4){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN4REG &= MAIN4GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED4ON;
			stagedOn |= STAGED4ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN4REG |= MAIN4GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED4ON){
			STAGEDPORT &= STAGED4OFF;
			stagedOn &= STAGED4OFF;
		}
	}
}

void Injector5ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN5;

	/* If rising edge triggered this */
	if(PTIT & MAIN5){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN5REG &= MAIN5GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED5ON;
			stagedOn |= STAGED5ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN5REG |= MAIN5GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED5ON){
			STAGEDPORT &= STAGED5OFF;
			stagedOn &= STAGED5OFF;
		}
	}
}

void Injector6ISR(void)
{
	/* Clear the interrupt flag for this channel */
	TFLG1 = MAIN6;

	/* If rising edge triggered this */
	if(PTIT & MAIN6){ // Stuff for switch on time
		/* Set the action for compare to switch off */
		MAIN6REG &= MAIN6GOLOW;
		
		/* Calculate off time based on on time and pulsewidth and set timer to switch off again. */
		// TODO calculate pulsewidth considering staged or not and total requirement of flow
		// TODO set timer compare value and set channel to turn off
		// TODO set stagedRequired if needed
		stagedRequired = 1; // temporary for testing.
		
		/* If staged injection is enabled, switch on corresponding staged injector and remember that we did. */
		if(stagedInjection && stagedRequired){
			STAGEDPORT |= STAGED6ON;
			stagedOn |= STAGED6ON;
		}
	}else{ // Stuff for switch off time
		/* Set the action for compare to switch on */
		MAIN6REG |= MAIN6GOHIGH;

		/* If we switched the staged injector on, turn it off now. */
		if(stagedOn & STAGED6ON){
			STAGEDPORT &= STAGED6OFF;
			stagedOn &= STAGED6OFF;
		}
	}
}
